#include <SPI.h>
/* Include Bluetooth modules */
#include "Adafruit_BLE.h"
#include "Adafruit_BluefruitLE_SPI.h"
#include "Adafruit_BluefruitLE_UART.h"
#include "BluefruitConfig.h"
/* Include accelerometer */
#include <Adafruit_MMA8451.h>
#include <Adafruit_Sensor.h>
/* Include SigFox library*/
#include <SigFox.h>
/* Include State Machine */
#include "src\StateMachine\single\TRSSyncSingleReceptor.h"
#include "src\StateMachine\single\MainSM.h"

#define LED_PIN 5 //just for an example

#define DEBUG_PRINT 
Adafruit_BluefruitLE_SPI ble(BLUEFRUIT_SPI_CS, BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);
// A small helper
void error(const __FlashStringHelper*err) {
  Serial.println(err);
  while (1);
}
Adafruit_MMA8451 mma = Adafruit_MMA8451();

void EnterCritical(void);
void ExitCritical(void);

/* Funct prototypes, maybe better to present in external headers */
bool is_bluetooth_present(void);
bool is_infant_present(void);
bool is_vehicle_moving(void);
/* Mem manager */
int32_t Mem_init(void);
bool Mem_checkCRC(void);


/*
STATE_POST = 0,
STATE_INIT,  
STATE_CONF_MISS,  
STATE_START,  
STATE_DRIVING,  
STATE_HALT,  Fermata  
STATE_STOP,  Sosta  
*/
evtState_t matrix[]={
  {SM_POST_OnEnter, SM_POST_OnExit, SM_POST},  /* Event Post */
  {SM_INIT_OnEnter, SM_INIT_OnExit, SM_INIT}, /* Event Init */
  {NULL, NULL, NULL}, /* TODO: Check configuration stored in Flash */
  {SM_START_OnEnter, SM_START_OnExit, SM_START}, /* Event START */
  {SM_DRIVING_OnEnter, SM_DRIVING_OnExit, SM_DRIVING}, /* Event Driving */
  {SM_HALT_OnEnter, SM_HALT_OnExit, SM_HALT}, /* Event Halt (Fermata)*/
  {SM_STOP_OnEnter, SM_STOP_OnExit, SM_STOP}, /* Event STOP (Sosta) */
  {NULL, NULL, NULL}
};

const int fsrPin = A3;
/* Setup:
 *  Configures PIN ports and intializes 
 *  main state machine. 
*/
void setup() {
    pinMode(LED_PIN,OUTPUT); //Configures the LED PIN
    pinMode(fsrPin, INPUT); //Configures the FORCE presence PIN
    TokenizerAsyncSingleReceptor_TurnOnEngine();
}

/* @PRE: To be called AFTER exiting state POST 
 *  Returns TRUE if the Bluetooth dongle is paired with a 
 *  mobile device. 
*/
bool is_bluetooth_present(void){
  bool retVal = false; 
  int32_t chars_id;
  /* Perform some kind of averaging? */
  ble.println("AT+BLEGETRSSI");    /* Send command */
  chars_id = ble.readline_parseInt(); /* Parse result */
  ble.waitForOK();         /* Command succesful?*/
  /* Check response status */
  if(chars_id < 0)
  {
#ifdef DEBUG_PRINT    
    Serial.println(chars_id);
#endif 
    retVal = true;  
  }
  return retVal; 
}

/* @PRE condition: After POST exiting. 
 *  Returns TRUE if the cell resistor detects a force major than a treshold. 
*/
bool is_infant_present(void){
  bool babyPresence = false;
  int32_t force = analogRead(fsrPin);  
  if (force > 100) {
    babyPresence = true;
#ifdef DEBUG_PRINT    
    Serial.println("Baby on board!");
#endif 
    } else{
       babyPresence = false;
#ifdef DEBUG_PRINT    
       Serial.println("No baby on board");
#endif 
     }
  Serial.println(force);
  return babyPresence;
}

uint32_t debugcounter = 0;
float ACC_MAX[]={0,0,0};
float ACC_min[]={0,0,0};

/* @PRE condition: To be called after POST exit. 
 * @return: TRUE if movement compatible with a vehicle movement is detected. 
 */  
bool is_vehicle_moving(void){
  sensors_event_t event;
    mma.getEvent(&event);
    float x,y,z = 0;
    x = event.acceleration.x;
    y = event.acceleration.y;
    z = event.acceleration.z;
 #ifdef DEBUG_PRINT    
    Serial.print("X:"); Serial.print(x); Serial.print("\t");
    Serial.print("Y:"); Serial.print(y); Serial.print("\t");
    Serial.print("Z:"); Serial.println(z);
 #endif 

  if(debugcounter<100){
    if(x>ACC_MAX[0])ACC_MAX[0]=x;
    if(y>ACC_MAX[1])ACC_MAX[1]=y;
    if(z>ACC_MAX[2])ACC_MAX[2]=z;
 
    if(x<ACC_min[0])ACC_min[0]=x;
    if(y<ACC_min[1])ACC_min[1]=y;
    if(z<ACC_min[2])ACC_min[2]=z;
    debugcounter++;
    return false;
    };

     if((x<ACC_MAX[0]*3 && x >ACC_min[0]*3)&&
        (y<ACC_MAX[1]*3 && y>ACC_min[1]*3))
     {
      return false;
    }else{
      return true;
      }
      //TODO: find a more suitable algo for movement detection. 
}

/* 
 *  @PRE condition: launch after TokenizerAsyncSingleReceptor_TurnOnEngine();
 * Infinite loop. First executes the state(s) handler(s),  
 * then calls the "getValuesFromDevices" function.  
 */
 
void loop() {
        TokenizerAsyncSingleReceptor_taskLoop(); 
}/* end loop */ 

/* 
 *  Passes the values measured from the ADAFRUIT compontents to the 
 *  tokenized State Machine. 
*/

int32_t getValuesFromDevices(TokenizerSyncSingleReceptor_t *pMsg){
  pMsg->is_moving = is_vehicle_moving();
  pMsg->is_paired = is_bluetooth_present();
  pMsg->is_present = is_infant_present();
  delay(100); /* One check every 100 ms*/
  return 0;
}

/* 
 *  IRQs callback and Enter and Exit Critical handling.
 *  TODO: check for the correct behaviour of enter and 
 *  exit critical.
*/
volatile bool TIM1_Elapsed;

#ifdef ARDUNO
ISR(TIMER1_COMPA_vect){
 TIM1_Elapsed = true;
}
#endif 

//IRQs for State Machine and MUTEX
void EnterCritical(void){
  noInterrupts();
}

void ExitCritical(void){
  interrupts();
}
