#ifndef __STATES_H
#define __STATES_H

/***************************************************************************/
/*POST State description:  
* On enter : never called. 
*/
void SM_POST_OnEnter(void);

/*
 * On exit: Should handle negative exit cases. 
 * TODO: Handle negative exit. 
*/
void SM_POST_OnExit(void);

/*
 * Checks Adafruit components: starts the turn on 
 * routine and checks for return values. 
 * If error blocks in a while loop.
 * TODO: Manage errors. 
*/
int32_t SM_POST(TokenizerSyncSingleReceptor_t *pMsg);
/***************************************************************************/

/*INIT State description:  
* On enter : Waits for bluetooth pairing. 
* Since it's called once, we invoke the function in the enter state. 
*/
void SM_INIT_OnEnter(void);

/*
 * On exit: Prints the exit condition. Just for debug now.  
 *  
*/
void SM_INIT_OnExit(void);

/*
 * Initializes values for ADA components:
 * Sets Accelerometer range, initializes timer(s).
 * TODO: Init SIGFox module here, get limits for static vehicle. 
*/
int32_t SM_INIT(TokenizerSyncSingleReceptor_t *pMsg);

/***************************************************************************/

/*
 *Does nothing, set for debug purposes 
*/
void SM_START_OnEnter(void);

/*
 *Does nothing, set for debug purposes 
*/
void SM_START_OnExit(void);

/*
 * Checks for infant presence, bluetooth pairing and and for movement. 
 * If not, waits until the three conditions become true. 
*/
int32_t SM_START(TokenizerSyncSingleReceptor_t *pMsg);

/***************************************************************************/

/*
 *Does nothing, set for debug purposes 
*/
void SM_DRIVING_OnEnter(void);

/*
 *Does nothing, set for debug purposes 
*/
void SM_DRIVING_OnExit(void);

/*
 * In this state the only significant parameter is the movment. 
 * If there is no movement, enters in the Halt state. 
 * If there is movement, keeps the current state. 
*/
int32_t SM_DRIVING(TokenizerSyncSingleReceptor_t *pMsg);

/***************************************************************************/

/*
 * Configures the tresholds(will be read from the structure stored in Flash) 
 * and sets the measurements parameters (SecondsElapsed; HaltTimerTresh;).
 * Start the timer for the state permanency measurement;
 * IRQ Based. 
 * 
*/
void SM_HALT_OnEnter(void);

/*
 * Stops and resets the timer.
*/
void SM_HALT_OnExit(void);

/*
 * Checks how long the system is in the Halt state and increases the timer counter. 
 * If it is kept for long enough(HaltTimerTresh) then it proceeds to the STOP state. 
 * Else keeps the current state. 
*/
int32_t SM_HALT(TokenizerSyncSingleReceptor_t *pMsg);

/***************************************************************************/

/*
 * Configures the tresholds(will be read from the structure stored in Flash) 
 * and sets the measurements parameters (STOP_TimerCounter, STOP_TimerCounter_tresh,
 * STOP_TimerTurnOff, STOP_TimerTurnOff_tresh). Sets also the alarm level 
 * Start the timer for the Alarm level measurment;
 * IRQ Based. 
 * 
*/
void SM_STOP_OnEnter(void);

/*
 * STOP state exits only with power off OR if the variable ispresent returns false. 
 * In that case the turn off pin is set as High. 
 * TODO: Add the turn off pin and an external transistor. 
*/
void SM_STOP_OnExit(void);

/*
 * If the variable ispresent is true, the track of the elapsed time is kept. 
 * Timer tresholds are set to progressively shift from an alarm level to the 
 * next one. The time between alarms is the STOP_TimerCounter_tresh.
 * When the maximum alarm level is reached, the system stays in a while loop. 
 * How the system should handle the maximum alarm case is to be defined. 
 * TODO: Handle the disarm event (if the APP from the mobile disarms the alarm
 * level in case of false positive events).
*/
int32_t SM_STOP(TokenizerSyncSingleReceptor_t *pMsg);

#endif //__STATES_H
