/**
 * Mutex.c
 * @brief 
 * @date   9 Oct 2017
 * @author Luca Toso
 * 
 * 
 * Modified by: $Author:$
 * 				$Revision:$
 * 				$Date:$
 */

/**
 * @addtogroup
 * @{
 */



/****************************** SYSTEM INCLUDES *******************************/

/****************************** PROJECT INCLUDES ******************************/
#include "Mutex.h"
#include "../SystemDefs.h"
/******************************* LOCAL TYPEDEF ********************************/

/**************************** LOCAL MACRO DEFINITION **************************/
volatile uint8_t SR_reg;               /* Current value of the FAULTMASK register */
volatile uint8_t SR_lock = 0x00U;      /* Lock */
extern void EnterCritical(void);
extern void ExitCritical(void);

/* Save status register and disable interrupts */
/**************************** LOCAL CONST DEFINITION **************************/

/***************************** LOCAL VARIABLES ********************************/
/************************** LOCAL FUNCTION PROTOTYPES *************************/

/************************** FUNCTIONS IMPLEMENTATION **************************/

mutex_t * Mutex_create(void){
	mutex_t* me = (mutex_t*)malloc(sizeof(mutex_t));
	if(NULL != me){
		Mutex_init(me);
	}
	return me;
}

mutex_status_t Mutex_init(mutex_t *pMutex)
{
    //assert(pMutex);

    pMutex->isLocked  = false;
    pMutex->isWaiting = false;
    pMutex->time_start = 0u;
    pMutex->wait_ms = 0u;

    return Mutex_Success;
}

mutex_status_t Mutex_lock(mutex_t *pMutex, uint32_t timeout)
{
	mutex_status_t retVal;
    //assert(pMutex);

    if (false == pMutex->isLocked )
    {
        /* Get the lock and return success */
        //INT_SYS_DisableIRQGlobal();
    	EnterCritical();
		pMutex->isLocked = true;
        pMutex->isWaiting = false;
        ExitCritical();
        //INT_SYS_EnableIRQGlobal();
        //return Mutex_Success;
        retVal = Mutex_Success;
    } else {/* TODO: Manage mutex locked. */
        if (0 == timeout)
        {
            //return Mutex_Tout;
        	retVal = Mutex_Tout;
        }
        else if (pMutex->isWaiting){
        	//check timeout here and set reval as Mutex_Tout;
        	retVal = Mutex_Tout;
        }

    }
    return retVal;
}

mutex_status_t Mutex_release(mutex_t *pMutex)
{
//    assert(pMutex);

    //INT_SYS_DisableIRQGlobal();
    EnterCritical();
    pMutex->isLocked = false;
    //INT_SYS_EnableIRQGlobal();
    ExitCritical();
    return Mutex_Success;
}

void DisableGlobalIRQ(void){

}

void Mutex_destroy(mutex_t* pMutex){
	if(NULL != pMutex){
		pMutex = NULL;
		free(pMutex);
	}

}

/** @}*/
/******************************** END OF FILE *********************************/
