/**
 * Mutex.h
 * @brief 
 * @date   9 Oct 2017
 * @author Luca Toso
 * 
 * 
 * Modified by: $Author:$
 * 				$Revision:$
 * 				$Date:$
 */

/**
 * @addtogroup
 * @{
 */



#ifndef STATEMACHINE_SINGLE_MUTEX_H_
#define STATEMACHINE_SINGLE_MUTEX_H_


/****************************** SYSTEM INCLUDES *******************************/
#include "../SystemDefs.h"
/******************************* PROJECT INCLUDES *****************************/

/************************* TYPEDEFS, STRUCTURES and UNIONS ********************/
typedef enum _mutex_status_t
{
    Mutex_Success = 0U,
    Mutex_Err,
    Mutex_Tout,
    Mutex_Wait
} mutex_status_t;

/*! @brief Type for a mutex */
typedef struct Mutex
{
    volatile bool    isWaiting;  /*!< Is any task waiting for a timeout on this mutex */
    volatile bool    isLocked;   /*!< Is the object locked or not                     */
    uint32_t         time_start; /*!< The time to start timeout                       */
    uint32_t         wait_ms;    /*!< Timeout to wait in milliseconds                 */
} mutex_t;
/************************ MACRO AND CONST DEFINITION **************************/
/************************** EXPOSED FUNCTION PROTOTYPES ***********************/
mutex_t * Mutex_create(void);
mutex_status_t Mutex_init(mutex_t *pMutex);
mutex_status_t Mutex_lock(mutex_t *pMutex, uint32_t timeout);
mutex_status_t Mutex_release(mutex_t *pMutex);
void Mutex_destroy(mutex_t *pMutex);





#endif /* STATEMACHINE_SINGLE_MUTEX_H_ */
/** @}*/
/******************************** END OF FILE *********************************/
