#include "TRSSyncSingleReceptor.h"

extern TSREventQueue_t * EVT_Queue;

static void cleanUpMutexRelations(TokenizerSyncSingleReceptor_t* const me);

StateType_t CurrentState = STATE_TOTAL;
void TokenizerSyncSingleReceptor_Init(TokenizerSyncSingleReceptor_t* const me) {
	me->is_moving = false;
	me->is_paired = false;
	me->is_present = false;
	me->itsMutex = NULL;
	me->stateID = STATE_POST;
}

extern evtState_t matrix[];

/* Mealy flawed state machine */
void TokenizerSyncSingleReceptor_eventDispatch(TokenizerSyncSingleReceptor_t* const me, Event_t e) {

	Mutex_lock(me->itsMutex, 0);
	CurrentState = me->stateID;
	if (NULL != matrix[CurrentState].pStateHandler) {
		/* Execute State handler */
		(matrix[CurrentState].pStateHandler)(me);
	}

	/* Enter in current state */
	if (CurrentState != me->stateID) {
		if (NULL != matrix[CurrentState].pOnExit) {
			(matrix[CurrentState].pOnExit)();
		}
	}
	/* If next state and current state differ, perform enter routine for new state*/
	if (CurrentState != me->stateID) {
		if (NULL != matrix[me->stateID].pOnEnter) {
			(matrix[me->stateID].pOnEnter)();
		}
	}
}

mutex_t* TokenizerSyncSingleReceptor_getItsMutex(
		const TokenizerSyncSingleReceptor_t* const me) {
	return (mutex_t*) me->itsMutex;
}

void TokenizerSyncSingleReceptor_setItsMutex(
		TokenizerSyncSingleReceptor_t* const me, mutex_t* pMutex) {
	me->itsMutex = pMutex;
}

TokenizerSyncSingleReceptor_t* TokenizerSyncSingleReceptor_Create(void) {
	TokenizerSyncSingleReceptor_t* me = (TokenizerSyncSingleReceptor_t*) malloc(
			sizeof(TokenizerSyncSingleReceptor_t*));
	if (NULL != me) {
		TokenizerSyncSingleReceptor_Init(me);
	}
	return me;
}

void TokenizerSyncSingleReceptor_Destroy(
		TokenizerSyncSingleReceptor_t* const me) {
	if (NULL != me) {
		TokenizerSyncSingleReceptor_Cleanup(me);
	}
	free(me);
}

void TokenizerSyncSingleReceptor_Cleanup(
		TokenizerSyncSingleReceptor_t* const me) {
	cleanUpMutexRelations(me);
}

static void cleanUpMutexRelations(TokenizerSyncSingleReceptor_t* const me) {
	if (NULL != me) {
		me->itsMutex = NULL;
	}
}

