#include "../SystemDefs.h"
#include "TSREventQueue.h"
#include "Mutex.h"

static void cleanUpRelations(TSREventQueue_t* const me);

void TSREventQueue_Init(TSREventQueue_t* const me){
    me->head = 0;
    me->size = 0;
    me->tail = 0;
    me->itsMutex = NULL;
}

void TSREventQueue_Cleanup(TSREventQueue_t *const me){
    cleanUpRelations(me);
}

int TSREventQueue_isEmpty(TSREventQueue_t *const me){
    return me->size <= 0;
}

int TSREventQueue_isFull(TSREventQueue_t* const me){
    return me->size >= QSIZE -1;
}

/* Event Queue, signals when an event is present to the OS/macroloop */
int TSREventQueue_post(TSREventQueue_t*const me, Event_t e){
   int retVal;
    Mutex_lock(me->itsMutex, 0);
    if(!TSREventQueue_isFull(me)){
        me->q[me->head] = e;
        me->head = (me->head+1)%QSIZE;
        me->size +=1;
        Mutex_release(me->itsMutex);
        postSignal();/* signal that an event is present*/
        retVal = 1;
    }else{
        Mutex_release(me->itsMutex);
        retVal = 0;
    }
    return retVal;
}
/*  pull removes the oldest event from the queue
    pull should only be called when there is an event waiting.
*/

Event_t TSREventQueue_pull(TSREventQueue_t* const me){
    Event_t e;
    Mutex_lock(me->itsMutex, 0);
    if(!TSREventQueue_isEmpty(me)){
        e = me->q[me->tail];
        me->tail = (me->tail+1)%QSIZE;
        me->size -= 1;
    }
    Mutex_release(me->itsMutex);
    return e;
}

struct Mutex* TSREventQueue_getItsMutex(const TSREventQueue_t* const me){
    return (mutex_t *)me->itsMutex;
};

void TSREventQueue_setItsMutex(TSREventQueue_t* const me, mutex_t* pMutex){
    me->itsMutex = pMutex;
};

TSREventQueue_t* TSREventQueue_Create(void){
    TSREventQueue_t *me = (TSREventQueue_t*)malloc(sizeof(TSREventQueue_t));
    if(me!=NULL){
        TSREventQueue_Init(me);
    }
    return me;
}

void TSREventQueue_Destroy(TSREventQueue_t *const me){
    if(me!=NULL){
        TSREventQueue_Cleanup(me);
    }
    free(me);
}

static void cleanUpRelations(TSREventQueue_t* const me){
if(me->itsMutex != NULL)
    me->itsMutex= NULL;
}

